@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2002-2014 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2002-2016 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
 
     Copyright (C) 2012-2015 by Khaled Hosny <khaledhosny@eglug.org>
@@ -60,7 +60,6 @@
 #include "dvi.h"
 #include "dvipdfmx.h"
 
-#ifdef XETEX
 #include "dpxfile.h"
 #include "pdfximage.h"
 #include "tt_aux.h"
@@ -68,7 +67,6 @@
 #include "t1_load.h"
 #include "t1_char.h"
 #include "cff_dict.h"
-#endif
 
 #define DVI_STACK_DEPTH_MAX  256u
 #define TEX_FONTS_ALLOC_SIZE 16u
@@ -145,20 +143,17 @@ static struct loaded_font
   int   tfm_id;
   spt_t size;
   int   source;     /* Source is either DVI or VF */
-#ifdef XETEX
   uint32_t rgba_color;
   struct tt_longMetrics *hvmt;
   int   ascent;
   int   descent;
   unsigned unitsPerEm;
   cff_font *cffont;
-  int cff_is_standard_encoding;
   unsigned numGlyphs;
   int   layout_dir;
   float extend;
   float slant;
   float embolden;
-#endif
 } *loaded_fonts = NULL;
 static int num_loaded_fonts = 0, max_loaded_fonts = 0;
 
@@ -179,7 +174,6 @@ static struct font_def
   char  *font_name;
   int    font_id;   /* index of _loaded_ font in loaded_fonts array */
   int    used;
-#ifdef XETEX
   int    native; /* boolean */
   uint32_t rgba_color;   /* only used for native fonts in XeTeX */
   uint32_t face_index;
@@ -187,17 +181,14 @@ static struct font_def
   int    extend;
   int    slant;
   int    embolden;
-#endif
 } *def_fonts = NULL;
 
-#ifdef XETEX
 #define XDV_FLAG_VERTICAL       0x0100
 #define XDV_FLAG_COLORED        0x0200
 #define XDV_FLAG_FEATURES       0x0400
 #define XDV_FLAG_EXTEND         0x1000
 #define XDV_FLAG_SLANT          0x2000
 #define XDV_FLAG_EMBOLDEN       0x4000
-#endif
 
 static int num_def_fonts = 0, max_def_fonts = 0;
 static int compute_boxes = 0, link_annot    = 1;
@@ -223,14 +214,12 @@ static int get_and_buffer_unsigned_byte (FILE *file)
   return ch;
 }
 
-#ifdef XETEX
 static unsigned int get_and_buffer_unsigned_pair (FILE *file)
 {
   unsigned int pair = get_and_buffer_unsigned_byte(file);
   pair = (pair << 8) | get_and_buffer_unsigned_byte(file);
   return pair;
 }
-#endif
 
 static void get_and_buffer_bytes(FILE *file, unsigned int count)
 {
@@ -250,14 +239,12 @@ static int get_buffered_unsigned_byte (void)
   return dvi_page_buffer[dvi_page_buf_index++];
 }
 
-#ifdef XETEX
 static unsigned int get_buffered_unsigned_pair (void)
 {
   unsigned int pair = dvi_page_buffer[dvi_page_buf_index++];
   pair = (pair << 8) | dvi_page_buffer[dvi_page_buf_index++];
   return pair;
 }
-#endif
 
 static int32_t get_buffered_signed_quad(void)
 {
@@ -326,6 +313,27 @@ static const char invalid_signature[] =
    ERROR(invalid_signature); \
  }
 
+static int pre_id_byte, post_id_byte, is_ptex = 0, has_ptex = 0;
+
+static void
+check_id_bytes (void) {
+  if (pre_id_byte != post_id_byte && (pre_id_byte != DVI_ID || post_id_byte != DVIV_ID))
+    ERROR ("Inconsistent DVI id_bytes %d (pre) and %d (post)", pre_id_byte, post_id_byte);
+}
+
+static void
+need_XeTeX (int c) {
+  if (!is_xdv)
+    ERROR ("DVI opcode %i only valid for XeTeX", c);
+}
+
+static void
+need_pTeX (int c) {
+  if (!is_ptex)
+    ERROR ("DVI opcode %i only valid for Ascii pTeX", c);
+  has_ptex = 1;
+}
+
 static int32_t
 find_post (void)
 {
@@ -349,12 +357,14 @@ find_post (void)
   /* file_position now points to last non padding character or
    * beginning of file */
   if (dvi_file_size - current < 4 || current == 0 ||
-      !(ch == DVI_ID || ch == DVIV_ID || ch == XDV_ID)) {
+      !(ch == DVI_ID || ch == DVIV_ID || ch == XDV_ID || ch == XDV_ID_OLD)) {
     MESG("DVI ID = %d\n", ch);
     ERROR(invalid_signature);
   } 
 
-  is_xdv = ch == XDV_ID;
+  post_id_byte = ch;
+  is_xdv = (ch == XDV_ID || ch == XDV_ID_OLD);
+  is_ptex = ch == DVIV_ID;
 
   /* Make sure post_post is really there */
   current = current - 5;
@@ -370,6 +380,21 @@ find_post (void)
     ERROR(invalid_signature);
   }
 
+  /* Finally check the ID byte in the preamble */
+  /* An Ascii pTeX DVI file has id_byte DVI_ID in the preamble but DVIV_ID in the postamble. */
+  xseek_absolute (dvi_file, 0, "DVI");
+  if ((ch = get_unsigned_byte(dvi_file)) != PRE) {
+    MESG("Found %d where PRE was expected\n", ch);
+    ERROR(invalid_signature);
+  }
+  ch = get_unsigned_byte(dvi_file);
+  if (!(ch == DVI_ID || ch == XDV_ID || ch == XDV_ID_OLD)) {
+    MESG("DVI ID = %d\n", ch);
+    ERROR(invalid_signature);
+  }
+  pre_id_byte = ch;
+  check_id_bytes();
+
   return current;
 }
 
@@ -457,13 +482,16 @@ get_preamble_dvi_info (void)
     ERROR(invalid_signature);
   }
   
+  /* An Ascii pTeX DVI file has id_byte DVI_ID in the preamble but DVIV_ID in the postamble. */
   ch = get_unsigned_byte(dvi_file);
-  if (!(ch == DVI_ID || ch == DVIV_ID || ch == XDV_ID)) {
+  if (!(ch == DVI_ID || ch == XDV_ID || ch == XDV_ID_OLD)) {
     MESG("DVI ID = %d\n", ch);
     ERROR(invalid_signature);
   }
 
-  is_xdv = ch == XDV_ID;
+  pre_id_byte = ch;
+  is_xdv = (ch == XDV_ID || ch == XDV_ID_OLD);
+  is_ptex = ch == DVI_ID; /* maybe */
   
   dvi_info.unit_num = get_positive_quad(dvi_file, "DVI", "unit_num");
   dvi_info.unit_den = get_positive_quad(dvi_file, "DVI", "unit_den");
@@ -529,7 +557,6 @@ read_font_record (int32_t tex_id)
   def_fonts[num_def_fonts].point_size  = point_size;
   def_fonts[num_def_fonts].design_size = design_size;
   def_fonts[num_def_fonts].used        = 0;
-#ifdef XETEX
   def_fonts[num_def_fonts].native      = 0;
   def_fonts[num_def_fonts].rgba_color  = 0xffffffff;
   def_fonts[num_def_fonts].face_index  = 0;
@@ -537,13 +564,11 @@ read_font_record (int32_t tex_id)
   def_fonts[num_def_fonts].extend      = 0x00010000; /* 1.0 */
   def_fonts[num_def_fonts].slant       = 0;
   def_fonts[num_def_fonts].embolden    = 0;
-#endif
   num_def_fonts++;
 
   return;
 }
 
-#ifdef XETEX
 static void
 read_native_font_record (int32_t tex_id)
 {
@@ -602,7 +627,6 @@ read_native_font_record (int32_t tex_id)
 
   return;
 }
-#endif
 
 static void
 get_dvi_fonts (int32_t post_location)
@@ -615,11 +639,10 @@ get_dvi_fonts (int32_t post_location)
     case FNT_DEF1: case FNT_DEF2: case FNT_DEF3: case FNT_DEF4:
       read_font_record(get_unsigned_num(dvi_file, code-FNT_DEF1));
       break;
-#ifdef XETEX
     case XDV_NATIVE_FONT_DEF:
+      need_XeTeX(code);
       read_native_font_record(get_signed_quad(dvi_file));
       break;
-#endif
     default:
       MESG("Unexpected op code: %3d\n", code);
       ERROR(invalid_signature);
@@ -919,7 +942,6 @@ dvi_locate_font (const char *tfm_name, spt_t ptsize)
   return  cur_id;
 }
 
-#ifdef XETEX
 static int
 dvi_locate_native_font (const char *filename, uint32_t index,
                         spt_t ptsize, int layout_dir, int extend, int slant, int embolden)
@@ -930,7 +952,7 @@ dvi_locate_native_font (const char *filename, uint32_t index,
   FILE         *fp;
   char         *path;
   sfnt         *sfont;
-  unsigned      offset = 0;
+  ULONG         offset = 0;
   struct tt_head_table *head;
   struct tt_maxp_table *maxp;
   struct tt_hhea_table *hhea;
@@ -986,7 +1008,6 @@ dvi_locate_native_font (const char *filename, uint32_t index,
       ERROR("Failed to read Type 1 font \"%s\".", filename);
 
     loaded_fonts[cur_id].cffont = cffont;
-    loaded_fonts[cur_id].cff_is_standard_encoding = enc_vec[0] == NULL;
 
     if (cff_dict_known(cffont->topdict, "FontBBox")) {
       loaded_fonts[cur_id].ascent = cff_dict_get(cffont->topdict, "FontBBox", 3);
@@ -1045,7 +1066,6 @@ dvi_locate_native_font (const char *filename, uint32_t index,
 
   return cur_id;
 }
-#endif
 
 double
 dvi_dev_xpos (void)
@@ -1452,7 +1472,6 @@ do_fnt (int32_t tex_id)
   if (!def_fonts[i].used) {
     int  font_id;
 
-#ifdef XETEX
     if (def_fonts[i].native) {
       font_id = dvi_locate_native_font(def_fonts[i].font_name,
                                        def_fonts[i].face_index,
@@ -1466,9 +1485,6 @@ do_fnt (int32_t tex_id)
                                 def_fonts[i].point_size);
     }
     loaded_fonts[font_id].rgba_color = def_fonts[i].rgba_color;
-#else
-    font_id = dvi_locate_font(def_fonts[i].font_name, def_fonts[i].point_size);
-#endif
     loaded_fonts[font_id].source = DVI;
     def_fonts[i].used    = 1;
     def_fonts[i].font_id = font_id;
@@ -1585,26 +1601,37 @@ dvi_end_reflect (void)
   }
 }
 
-#ifdef XETEX
+static void
+skip_native_font_def (void)
+{
+  unsigned int flags;
+  int name_length;
+
+  skip_bytes(4, dvi_file); /* skip point size */
+  flags = get_unsigned_pair(dvi_file);
+  name_length = get_unsigned_byte(dvi_file);
+  skip_bytes(name_length + 4, dvi_file);
+
+  if (flags & XDV_FLAG_COLORED)
+    skip_bytes(4, dvi_file);
+
+  if (flags & XDV_FLAG_EXTEND)
+    skip_bytes(4, dvi_file);
+
+  if (flags & XDV_FLAG_SLANT)
+    skip_bytes(4, dvi_file);
+
+  if (flags & XDV_FLAG_EMBOLDEN)
+    skip_bytes(4, dvi_file);
+}
+
 static void
 do_native_font_def (int32_t tex_id)
 {
-  if (linear) {
+  if (linear)
     read_native_font_record(tex_id);
-  } else {
-    unsigned int flags;
-    int name_length, i;
-
-    get_unsigned_quad(dvi_file); /* skip point size */
-    flags = get_unsigned_pair(dvi_file);
-    name_length = (int) get_unsigned_byte(dvi_file);
-    for (i = 0; i < name_length; ++i)
-      get_unsigned_byte(dvi_file);
-    get_unsigned_quad(dvi_file);
-    if (flags & XDV_FLAG_COLORED) {
-      get_unsigned_quad(dvi_file);
-    }
-  }
+  else
+    skip_native_font_def();
   --dvi_page_buf_index; /* don't buffer the opcode */
 }
 
@@ -1621,7 +1648,7 @@ skip_glyphs (void)
 }
 
 static void
-do_glyphs (void)
+do_glyphs (int do_actual_text)
 {
   struct loaded_font *font;
   spt_t  width, height, depth, *xloc, *yloc, glyph_width = 0;
@@ -1633,6 +1660,22 @@ do_glyphs (void)
 
   font  = &loaded_fonts[current_font];
 
+  if (do_actual_text) {
+    slen = (unsigned int) get_buffered_unsigned_pair();
+    if (lr_mode >= SKIMMING) {
+      for (i = 0; i < slen; i++) {
+        skip_bufferd_bytes(2);
+      }
+    } else {
+      uint16_t *unicodes = NEW(slen, uint16_t);
+      for (i = 0; i < slen; i++) {
+        unicodes[i] = (uint16_t) get_buffered_unsigned_pair();
+      }
+      pdf_dev_begin_actualtext (unicodes, slen);
+      RELEASE(unicodes);
+    }
+  }
+
   width = get_buffered_signed_quad();
 
   if (lr_mode >= SKIMMING) {
@@ -1672,8 +1715,9 @@ do_glyphs (void)
         cff_index *cstrings = font->cffont->cstrings;
         t1_ginfo gm;
 
-        /* For standard encoding, Type1 glyph id is FreeType glyph index + 1. */
-        if (font->cff_is_standard_encoding)
+        /* If .notdef is not the 1st glyph in CharStrings, glyph_id given by
+           FreeType should be increased by 1 */
+        if (font->cffont->is_notdef_notzero)
           glyph_id += 1;
 
         t1char_get_metrics(cstrings->data + cstrings->offset[glyph_id] - 1,
@@ -1711,20 +1755,48 @@ do_glyphs (void)
   RELEASE(xloc);
   RELEASE(yloc);
 
+  if (do_actual_text) {
+    pdf_dev_end_actualtext();
+  }
+
   if (lr_mode == LTYPESETTING)
     dvi_right(width);
 
   return;
 }
-#endif
 
-/* Note to be absolutely certain that the string escape buffer doesn't
- * hit its limit, FORMAT_BUF_SIZE should set to 4 times S_BUFFER_SIZE
- * in pdfobj.c.  Is there any application that genenerate words with
- * 1k characters?
- */
+static void
+check_postamble (void)
+{
+  int code;
 
-#define SBUF_SIZE 1024
+  skip_bytes(28, dvi_file);
+  while ((code = get_unsigned_byte(dvi_file)) != POST_POST) {
+    switch (code) {
+    case FNT_DEF1: case FNT_DEF2: case FNT_DEF3: case FNT_DEF4:
+      skip_bytes(code + 1 - FNT_DEF1, dvi_file);
+      skip_fntdef();
+      break;
+    case XDV_NATIVE_FONT_DEF:
+      skip_bytes(4, dvi_file);
+      skip_native_font_def();
+      break;
+    default:
+      ERROR("Unexpected op code (%u) in postamble", code);
+    }
+  }
+  skip_bytes(4, dvi_file);
+  post_id_byte = get_unsigned_byte(dvi_file);
+  if (!(post_id_byte == DVI_ID || post_id_byte == DVIV_ID || post_id_byte == XDV_ID || post_id_byte == XDV_ID_OLD)) {
+    MESG("DVI ID = %d\n", post_id_byte);
+    ERROR(invalid_signature);
+  }
+  check_id_bytes();
+  if (has_ptex && post_id_byte != DVIV_ID)
+    ERROR ("Saw opcode %i in DVI file not for Ascii pTeX", PTEXDIR);
+
+  num_pages = 0; /* force loop to terminate */
+}
 
 /* Most of the work of actually interpreting
  * the dvi file is here.
@@ -1788,6 +1860,12 @@ dvi_do_page (double page_paper_height, double hmargin, double vmargin)
       break;
     case EOP:
       do_eop();
+      if (linear) {
+        if ((opcode = get_unsigned_byte(dvi_file)) == POST)
+          check_postamble();
+        else
+          ungetc(opcode, dvi_file);
+      }
       return;
 
     case PUSH:
@@ -1854,22 +1932,29 @@ dvi_do_page (double page_paper_height, double hmargin, double vmargin)
 
       /* pTeX extension */
     case PTEXDIR:
+      need_pTeX(opcode);
       do_dir();
       break;
 
-#ifdef XETEX
-    /* XeTeX extension */
+    /* XeTeX extensions */
     case XDV_GLYPHS:
-      do_glyphs();
+      need_XeTeX(opcode);
+      do_glyphs(0);
+      break;
+    case XDV_TEXT_AND_GLYPHS:
+      need_XeTeX(opcode);
+      do_glyphs(1);
       break;
     /* should not occur - processed during pre-scanning */
     case XDV_NATIVE_FONT_DEF:
+      need_XeTeX(opcode);
       break;
-#endif
     case BEGIN_REFLECT:
+      need_XeTeX(opcode);
       dvi_begin_reflect();
       break;
     case END_REFLECT:
+      need_XeTeX(opcode);
       dvi_end_reflect();
       break;
 
@@ -1895,6 +1980,7 @@ dvi_init (char *dvi_filename, double mag)
   int32_t post_location;
 
   if (!dvi_filename) { /* no filename: reading from stdin, probably a pipe */
+    int ch;
 #ifdef WIN32
         setmode(fileno(stdin), _O_BINARY);
 #endif
@@ -1903,6 +1989,10 @@ dvi_init (char *dvi_filename, double mag)
 
     get_preamble_dvi_info();
     do_scales(mag);
+    if ((ch = get_unsigned_byte(dvi_file)) == POST)
+      check_postamble();
+    else
+      ungetc(ch, dvi_file);
   } else {
     dvi_file = MFOPEN(dvi_filename, FOPEN_RBIN_MODE);
     if (!dvi_file) {
@@ -1978,7 +2068,6 @@ dvi_close (void)
   page_loc  = NULL;
   num_pages = 0;
 
-#ifdef XETEX
   for (i = 0; i < num_loaded_fonts; i++)
   {
     if (loaded_fonts[i].hvmt != NULL)
@@ -1991,7 +2080,6 @@ dvi_close (void)
 
     loaded_fonts[i].cffont = NULL;
   }
-#endif
 
   if (loaded_fonts)
     RELEASE(loaded_fonts);
@@ -2120,14 +2208,15 @@ read_length (double *vp, double mag, const char **pp, const char *endptr)
 
 
 static int
-scan_special (double *wd, double *ht, double *xo, double *yo, char *lm,
-              unsigned *minorversion,
-              int *do_enc, unsigned *key_bits, unsigned *permission, char *owner_pw, char *user_pw,
+scan_special (double *wd, double *ht, double *xo, double *yo, int *lm,
+              int *majorversion, int *minorversion,
+              int *do_enc, int *key_bits, int32_t *permission,
+              char *owner_pw, char *user_pw,
               const char *buf, uint32_t size)
 {
   char  *q;
   const char *p = buf, *endptr;
-  int    ns_pdf = 0, error = 0;
+  int    ns_pdf = 0, ns_dvipdfmx = 0, error = 0;
   double tmp;
 
   endptr = p + size;
@@ -2153,6 +2242,15 @@ scan_special (double *wd, double *ht, double *xo, double *yo, char *lm,
       q = parse_c_ident(&p, endptr);
     }
   }
+  else if (q && !strcmp(q, "dvipdfmx")) {
+    skip_white(&p, endptr);
+    if (p < endptr && *p == ':') {
+      p++;
+      skip_white(&p, endptr);
+      RELEASE(q);
+      q = parse_c_ident(&p, endptr); ns_dvipdfmx = 1;
+    }
+  }
   skip_white(&p, endptr);
 
   if (q) {
@@ -2227,7 +2325,16 @@ scan_special (double *wd, double *ht, double *xo, double *yo, char *lm,
       skip_white(&p, endptr);
       kv = parse_float_decimal(&p, endptr);
       if (kv) {
-        *minorversion = (unsigned)strtol(kv, NULL, 10);
+        *minorversion = (int)strtol(kv, NULL, 10);
+        RELEASE(kv);
+      }
+    } else if (majorversion && ns_pdf && !strcmp(q, "majorversion")) {
+      char *kv;
+      if (*p == '=') p++;
+      skip_white(&p, endptr);
+      kv = parse_float_decimal(&p, endptr);
+      if (kv) {
+        *majorversion = (int)strtol(kv, NULL, 10);
         RELEASE(kv);
       }
     } else if (ns_pdf && !strcmp(q, "encrypt") && do_enc) {
@@ -2242,13 +2349,15 @@ scan_special (double *wd, double *ht, double *xo, double *yo, char *lm,
           skip_white(&p, endptr);
           if (!strcmp(kp, "ownerpw")) {
             if ((obj = parse_pdf_string(&p, endptr))) {
-              strncpy(owner_pw, pdf_string_value(obj), MAX_PWD_LEN); 
+              if (pdf_string_value(obj))
+                strncpy(owner_pw, pdf_string_value(obj), MAX_PWD_LEN);
               pdf_release_obj(obj);
             } else
               error = -1;
           } else if (!strcmp(kp, "userpw")) {
             if ((obj = parse_pdf_string(&p, endptr))) {
-              strncpy(user_pw, pdf_string_value(obj), MAX_PWD_LEN);
+              if (pdf_string_value(obj))
+                strncpy(user_pw, pdf_string_value(obj), MAX_PWD_LEN);
               pdf_release_obj(obj);
             } else
               error = -1;
@@ -2272,6 +2381,8 @@ scan_special (double *wd, double *ht, double *xo, double *yo, char *lm,
         }
         skip_white(&p, endptr);
       }
+    } else if (ns_dvipdfmx && !strcmp(q, "config")) {
+      read_config_special(&p, endptr);
     }
     RELEASE(q);
   }
@@ -2283,19 +2394,18 @@ scan_special (double *wd, double *ht, double *xo, double *yo, char *lm,
 void
 dvi_scan_specials (int page_no,
                    double *page_width, double *page_height,
-                   double *x_offset, double *y_offset, char *landscape,
-                   unsigned *minorversion,
-                   int *do_enc, unsigned *key_bits, unsigned *permission, char *owner_pw, char *user_pw)
+                   double *x_offset, double *y_offset, int *landscape,
+                   int *majorversion, int *minorversion,
+                   int *do_enc, int *key_bits, int32_t *permission,
+                   char *owner_pw, char *user_pw)
 {
   FILE          *fp = dvi_file;
   int32_t        offset;
   unsigned char  opcode;
   static int     buffered_page = -1;
-#if XETEX
   unsigned int len;
-#endif
 
-  if (page_no == buffered_page)
+  if (page_no == buffered_page || num_pages == 0)
     return; /* because dvipdfmx wants to scan first page twice! */
   buffered_page = page_no;
 
@@ -2328,11 +2438,12 @@ dvi_scan_specials (int page_no,
         dvi_page_buf_size = (dvi_page_buf_index + size + DVI_PAGE_BUF_CHUNK);
         dvi_page_buffer = RENEW(dvi_page_buffer, dvi_page_buf_size, unsigned char);
       }
-#define buf (char*)dvi_page_buffer + dvi_page_buf_index
+#define buf ((char*)(dvi_page_buffer + dvi_page_buf_index))
       if (fread(buf, sizeof(char), size, fp) != size)
         ERROR("Reading DVI file failed!");
       if (scan_special(page_width, page_height, x_offset, y_offset, landscape,
-                       minorversion, do_enc, key_bits, permission, owner_pw, user_pw,
+                       majorversion, minorversion,
+                       do_enc, key_bits, permission, owner_pw, user_pw,
                        buf, size))
         WARN("Reading special command failed: \"%.*s\"", size, buf);
 #undef buf
@@ -2375,21 +2486,31 @@ dvi_scan_specials (int page_no,
     case FNT_DEF1: case FNT_DEF2: case FNT_DEF3: case FNT_DEF4:
       do_fntdef(get_unsigned_num(fp, opcode-FNT_DEF1));
       break;
-#ifdef XETEX
     case XDV_GLYPHS:
+      need_XeTeX(opcode);
+      get_and_buffer_bytes(fp, 4);            /* width */
+      len = get_and_buffer_unsigned_pair(fp); /* glyph count */
+      get_and_buffer_bytes(fp, len * 10);     /* 2 bytes ID + 8 bytes x,y-location per glyph */
+      break;
+    case XDV_TEXT_AND_GLYPHS:
+      need_XeTeX(opcode);
+      len = get_and_buffer_unsigned_pair(fp); /* utf16 code unit count */
+      get_and_buffer_bytes(fp, len * 2);      /* 2 bytes per code unit */
       get_and_buffer_bytes(fp, 4);            /* width */
       len = get_and_buffer_unsigned_pair(fp); /* glyph count */
       get_and_buffer_bytes(fp, len * 10);     /* 2 bytes ID + 8 bytes x,y-location per glyph */
       break;
     case XDV_NATIVE_FONT_DEF:
+      need_XeTeX(opcode);
       do_native_font_def(get_signed_quad(dvi_file));
       break;
-#endif
     case BEGIN_REFLECT:
     case END_REFLECT:
+      need_XeTeX(opcode);
       break;
 
     case PTEXDIR:
+      need_pTeX(opcode);
       get_and_buffer_bytes(fp, 1);
       break;
 
@@ -2407,4 +2528,3 @@ dvi_scan_specials (int page_no,
 
   return;
 }
-
