@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2002-2015 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2002-2016 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -76,7 +76,7 @@ pdf_dev_scale (void)
 #define TEX_ONE_HUNDRED_BP 6578176
 static struct {
   double dvi2pts;
-  long   min_bp_val; /* Shortest resolvable distance in the output PDF.     */
+  int    min_bp_val; /* Shortest resolvable distance in the output PDF.     */
   int    precision;  /* Number of decimal digits (in fractional part) kept. */
 } dev_unit = {
   0.0,
@@ -92,8 +92,8 @@ dev_unit_dviunit (void)
 }
 
 #define DEV_PRECISION_MAX  8
-static unsigned long ten_pow[10] = {
-  1ul, 10ul, 100ul, 1000ul, 10000ul, 100000ul, 1000000ul, 10000000ul, 100000000ul, 1000000000ul
+static uint32_t ten_pow[10] = {
+  1u, 10u, 100u, 1000u, 10000u, 100000u, 1000000u, 10000000u, 100000000u, 1000000000u
 };
 
 static double ten_pow_inv[10] = {
@@ -105,7 +105,7 @@ static double ten_pow_inv[10] = {
 #define dround_at(v,p) (ROUND( (v), ten_pow_inv[(p)] ))
 
 static int
-p_itoa (long value, char *buf)
+p_itoa (int value, char *buf)
 {
   int   sign, ndigits;
   char *p = buf;
@@ -146,11 +146,11 @@ p_itoa (long value, char *buf)
 static int
 p_dtoa (double value, int prec, char *buf)
 {
-  const long p[10] = { 1, 10, 100, 1000, 10000,
+  const int32_t p[10] = { 1, 10, 100, 1000, 10000,
 		                   100000, 1000000, 10000000,
                        100000000, 1000000000 };
   double i, f;
-  long   g;
+  int32_t g;
   char  *c = buf;
   int    n;
 
@@ -163,7 +163,7 @@ p_dtoa (double value, int prec, char *buf)
   }
 
   f = modf(value, &i);
-  g = (long) (f * p[prec] + 0.5);
+  g = (int32_t) (f * p[prec] + 0.5);
 
   if (g == p[prec]) {
     g  = 0;
@@ -1045,14 +1045,10 @@ handle_multibyte_string (struct dev_font *font,
    * encoding.
    * TODO: A character decomposed to multiple characters.
    */
-#ifdef XETEX
   if (ctype != -1 && font->enc_id >= 0) {
-#else
-  if (font->enc_id >= 0) {
-#endif
     const unsigned char *inbuf;
     unsigned char *outbuf;
-    long           inbytesleft, outbytesleft;
+    int            inbytesleft, outbytesleft;
     CMap          *cmap;
 
     cmap         = CMap_cache_get(font->enc_id);
@@ -1109,9 +1105,8 @@ void pdf_dev_pop_coord(void)
 
 /*
  * ctype:
-#ifdef XETEX
  *  -1 input string contains 2-byte Freetype glyph index values
-#endif
+ *     (XeTeX only)
  *  0  byte-width of char can be variable and input string
  *     is properly encoded.
  *  n  Single character cosumes n bytes in input string.
@@ -1292,7 +1287,7 @@ pdf_init_device (double dvi2pts, int precision, int black_and_white)
     dev_unit.precision  = precision;
   }
   dev_unit.dvi2pts      = dvi2pts;
-  dev_unit.min_bp_val   = (long) ROUND(1.0/(ten_pow[dev_unit.precision]*dvi2pts), 1);
+  dev_unit.min_bp_val   = (int) ROUND(1.0/(ten_pow[dev_unit.precision]*dvi2pts), 1);
   if (dev_unit.min_bp_val < 0)
     dev_unit.min_bp_val = -dev_unit.min_bp_val;
 
@@ -1891,7 +1886,6 @@ pdf_dev_put_image (int             id,
                             res_name,
                             pdf_ximage_get_reference(id));
 
-#ifdef XETEX
   if (dvi_is_tracking_boxes()) {
     pdf_tmatrix P;
     int i;
@@ -1932,7 +1926,6 @@ pdf_dev_put_image (int             id,
 
     pdf_doc_expand_box(&rect);
   }
-#endif
 
   return 0;
 }
@@ -1956,3 +1949,56 @@ transform_info_clear (transform_info *info)
 
   info->flags    = 0;
 }
+
+void
+pdf_dev_begin_actualtext (uint16_t *unicodes, int count)
+{
+  int len, i, pdf_doc_enc = 1;
+
+  /* check whether we can use PDFDocEncoding for this string
+     (we punt on the 0x80..0xA0 range that does not directly correspond to unicode)  */
+  for (i = 0; i < count; i++) {
+    if (unicodes[i] > 0xff || (unicodes[i] > 0x7f && unicodes[i] < 0xa1)) {
+      pdf_doc_enc = 0;
+      break;
+    }
+  }
+
+  graphics_mode();
+
+  len = sprintf(work_buffer, "\n/Span<</ActualText(");
+  if (!pdf_doc_enc) {
+    len += sprintf(work_buffer + len, "\xFE\xFF");
+  }
+  pdf_doc_add_page_content(work_buffer, len);
+
+  while (count-- > 0) {
+    unsigned char s[2] = { *unicodes >> 8, *unicodes & 0xff };
+    i = pdf_doc_enc; /* if using PDFDocEncoding, we only care about the low 8 bits,
+                        so start with the second byte of our pair */
+    len = 0;
+    for (; i < 2; i++) {
+      unsigned char c = s[i];
+      if (c == '(' || c == ')' || c == '\\') {
+        len += sprintf(work_buffer + len, "\\%c", c);
+      } else if (c < ' ') {
+        len += sprintf(work_buffer + len, "\\%03o", c);
+      } else {
+        len += sprintf(work_buffer + len, "%c", c);
+      }
+    }
+    pdf_doc_add_page_content(work_buffer, len);
+    ++unicodes;
+  }
+
+  len = sprintf(work_buffer, ")>>BDC");
+  pdf_doc_add_page_content(work_buffer, len);
+}
+
+void
+pdf_dev_end_actualtext ()
+{
+  graphics_mode();
+
+  pdf_doc_add_page_content(" EMC", 4);
+}
