@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2002-2014 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2002-2016 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -55,7 +55,7 @@
 struct spc_html_
 {
   struct {
-    long  extensions;
+    int  extensions;
   } opts;
 
   pdf_obj  *link_dict;
@@ -570,6 +570,7 @@ spc_html__img_empty (struct spc_env *spe, pdf_obj *attr)
 {
   pdf_obj       *src, *obj;
   transform_info ti;
+  load_options   options = {1, 0, NULL};
   int            id, error = 0;
 #ifdef  ENABLE_HTML_SVG_OPACITY
   double         alpha = 1.0; /* meaning fully opaque */
@@ -636,7 +637,7 @@ spc_html__img_empty (struct spc_env *spe, pdf_obj *attr)
     return  error;
   }
 
-  id = pdf_ximage_findresource(pdf_string_value(src), 0, NULL);
+  id = pdf_ximage_findresource(pdf_string_value(src), options);
   if (id < 0) {
     spc_warn(spe, "Could not find/load image: %s", pdf_string_value(src)); 
     error = -1;
@@ -899,7 +900,7 @@ spc_html_at_end_document (void)
 
 
 int
-spc_html_check_special (const char *buffer, long size)
+spc_html_check_special (const char *buffer, int size)
 {
   const char *p, *endptr;
 
@@ -907,7 +908,7 @@ spc_html_check_special (const char *buffer, long size)
   endptr = p + size;
 
   for ( ; p < endptr && isspace((unsigned char)*p); p++);
-  size   = (long) (endptr - p);
+  size   = (int) (endptr - p);
   if (size >= strlen("html:") &&
       !memcmp(p, "html:", strlen("html:"))) {
     return  1;
@@ -939,4 +940,3 @@ spc_html_setup_handler (struct spc_handler *sph,
 
   return  0;
 }
-
