@@ -1,6 +1,6 @@
 /* mingw32.c: bits and pieces for mingw32
 
-   Copyright 2009-2013 Taco Hoekwater <taco@luatex.org>.
+   Copyright 2009-2016 Taco Hoekwater <taco@luatex.org>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -48,6 +48,12 @@ extern int __cdecl _free_osfhnd (int fd);
 static char *get_home_directory (void);
 static int _parse_root (char * name, char ** pPath);
 
+double
+win32_floor (double x)
+{
+  return floor (x);
+}
+
 void
 init_user_info (void)
 {
