@@ -20,6 +20,7 @@
 
 #include "ptexlib.h"
 #include "lua/luatex-api.h"
+#include "pdf/pdftables.h"
 
 static int luapdfprint(lua_State * L)
 {
@@ -29,52 +30,49 @@ static int luapdfprint(lua_State * L)
     ctm_transform_modes literal_mode;
     st.s = modestr_s = NULL;
     n = lua_gettop(L);
-    if (!lua_isstring(L, -1)) {
+    if (!lua_isstring(L, -1)) { /* or number */
         luaL_error(L, "no string to print");
     }
     literal_mode = set_origin;
     if (n == 2) {
-        if (!lua_isstring(L, -2)) {
-            luaL_error(L, "invalid argument for print literal mode");
+        if (lua_type(L,-2) != LUA_TSTRING) {
+            luaL_error(L, "invalid first argument for print literal mode");
         } else {
-	    modestr_s = lua_tostring(L, -2);
-	    if (lua_key_eq(modestr_s,direct))
-	      literal_mode = direct_always;
-	    else if (lua_key_eq(modestr_s,page))
+            modestr_s = lua_tostring(L, -2);
+            if (lua_key_eq(modestr_s,direct))
+                literal_mode = direct_always;
+            else if (lua_key_eq(modestr_s,page))
                 literal_mode = direct_page;
             else {
-                luaL_error(L, "invalid argument for print literal mode");
+                luaL_error(L, "invalid first argument for print literal mode");
             }
         }
-    } else {
-        if (n != 1) {
-            luaL_error(L, "invalid number of arguments");
-        }
+    } else if (n != 1) {
+        luaL_error(L, "invalid number of arguments");
     }
     check_o_mode(static_pdf, "pdf.print()", 1 << OMODE_PDF, true);
     switch (literal_mode) {
-    case (set_origin):
-        pdf_goto_pagemode(static_pdf);
-        pdf_set_pos(static_pdf, static_pdf->posstruct->pos);
-        (void) calc_pdfpos(static_pdf->pstruct, static_pdf->posstruct->pos);
-        break;
-    case (direct_page):
-        pdf_goto_pagemode(static_pdf);
-        (void) calc_pdfpos(static_pdf->pstruct, static_pdf->posstruct->pos);
-        break;
-    case (direct_always):
-        pdf_end_string_nl(static_pdf);
-        break;
-    default:
-        assert(0);
+        case (set_origin):
+            pdf_goto_pagemode(static_pdf);
+            pdf_set_pos(static_pdf, static_pdf->posstruct->pos);
+            (void) calc_pdfpos(static_pdf->pstruct, static_pdf->posstruct->pos);
+            break;
+        case (direct_page):
+            pdf_goto_pagemode(static_pdf);
+            (void) calc_pdfpos(static_pdf->pstruct, static_pdf->posstruct->pos);
+            break;
+        case (direct_always):
+            pdf_end_string_nl(static_pdf);
+            break;
+        default:
+            assert(0);
     }
     st.s = lua_tolstring(L, n, &st.l);
     pdf_out_block(static_pdf, st.s, st.l);
     return 0;
 }
 
-static unsigned char *fread_to_buf(lua_State * L, const char *filename,
-                                   size_t * len)
+static unsigned char *fread_to_buf(lua_State * L, const char *filename, size_t * len)
 {
     int ilen = 0;
     FILE *f;
@@ -100,9 +98,9 @@ static int l_immediateobj(lua_State * L)
     if (global_shipping_mode != NOT_SHIPPING)
         luaL_error(L, "pdf.immediateobj() can not be used with \\latelua");
     n = lua_gettop(L);
-    if (n > 0 && lua_type(L, 1) == LUA_TNUMBER) {
+    if ((n > 0) && (lua_type(L, 1) == LUA_TNUMBER)) {
         first_arg++;
-        k=(int)lua_tonumber(L, 1);
+        k = (int) lua_tointeger(L, 1);
         check_obj_type(static_pdf, obj_type_obj, k);
         if (is_obj_scheduled(static_pdf, k) || obj_data_ptr(static_pdf, k) != 0)
             luaL_error(L, "pdf.immediateobj() object in use");
@@ -112,69 +110,61 @@ static int l_immediateobj(lua_State * L)
     }
     pdf_last_obj = k;
     switch (n - first_arg + 1) {
-    case 0:
-        luaL_error(L, "pdf.immediateobj() needs at least one argument");
-        break;
-    case 1:
-        if (!lua_isstring(L, first_arg))
-            luaL_error(L, "pdf.immediateobj() 1st argument must be string");
-        pdf_begin_obj(static_pdf, k, OBJSTM_ALWAYS);
-        st1.s = lua_tolstring(L, first_arg, &st1.l);
-        pdf_out_block(static_pdf, st1.s, st1.l);
-        /* already in pdf_end_obj:
-            if (st1.s[st1.l - 1] != '\n')
-                pdf_out(static_pdf, '\n');
-        */
-        pdf_end_obj(static_pdf);
-        break;
-    case 2:
-    case 3:
-        if (!lua_isstring(L, first_arg))
-            luaL_error(L, "pdf.immediateobj() 1st argument must be string");
-        if (!lua_isstring(L, first_arg + 1))
-            luaL_error(L, "pdf.immediateobj() 2nd argument must be string");
-        st1_s = lua_tostring(L, first_arg);
-        st2.s = lua_tolstring(L, first_arg + 1, &st2.l);
-        if (lua_key_eq(st1_s, file)) {
-            if (n == first_arg + 2)
-                luaL_error(L, "pdf.immediateobj() 3rd argument forbidden in file mode");
+        case 0:
+            luaL_error(L, "pdf.immediateobj() needs at least one argument");
+            break;
+        case 1:
+            if (!lua_isstring(L, first_arg)) /* or number */
+                luaL_error(L, "pdf.immediateobj() 1st argument must be string");
             pdf_begin_obj(static_pdf, k, OBJSTM_ALWAYS);
-            buf.s = fread_to_buf(L, st2.s, &buf.l);
-            pdf_out_block(static_pdf, (const char *) buf.s, buf.l);
-            /* already in pdf_end_obj:
-                if (buf.s[buf.l - 1] != '\n')
-                    pdf_out(static_pdf, '\n');
-            */
-            xfree(buf.s);
+            st1.s = lua_tolstring(L, first_arg, &st1.l);
+            pdf_out_block(static_pdf, st1.s, st1.l);
             pdf_end_obj(static_pdf);
-        } else {
-            pdf_begin_obj(static_pdf, k, OBJSTM_NEVER); /* not an object stream candidate! */
-            pdf_begin_dict(static_pdf);
-            if (n == first_arg + 2) {   /* write attr text */
-                if (!lua_isstring(L, first_arg + 2))
-                    luaL_error(L, "pdf.immediateobj() 3rd argument must be string");
-                st3.s = lua_tolstring(L, first_arg + 2, &st3.l);
-                pdf_out_block(static_pdf, st3.s, st3.l);
-                if (st3.s[st3.l - 1] != '\n')
-                    pdf_out(static_pdf, '\n');
-            }
-            pdf_dict_add_streaminfo(static_pdf);
-            pdf_end_dict(static_pdf);
-            pdf_begin_stream(static_pdf);
-            if (lua_key_eq(st1_s, stream)) {
-                pdf_out_block(static_pdf, st2.s, st2.l);
-            }  else if (lua_key_eq(st1_s, streamfile)) {
+            break;
+        case 2:
+        case 3:
+            if (lua_type(L,first_arg) != LUA_TSTRING)
+                luaL_error(L, "pdf.immediateobj() 1st argument must be string");
+            if (!lua_isstring(L, first_arg + 1)) /* or number */
+                luaL_error(L, "pdf.immediateobj() 2nd argument must be string");
+            st1_s = lua_tostring(L, first_arg);
+            st2.s = lua_tolstring(L, first_arg + 1, &st2.l);
+            if (lua_key_eq(st1_s, file)) {
+                if (n == first_arg + 2)
+                    luaL_error(L, "pdf.immediateobj() 3rd argument forbidden in file mode");
+                pdf_begin_obj(static_pdf, k, OBJSTM_ALWAYS);
                 buf.s = fread_to_buf(L, st2.s, &buf.l);
                 pdf_out_block(static_pdf, (const char *) buf.s, buf.l);
                 xfree(buf.s);
-            } else
-                luaL_error(L, "pdf.immediateobj() invalid argument");
-            pdf_end_stream(static_pdf);
-            pdf_end_obj(static_pdf);
-        }
-        break;
-    default:
-        luaL_error(L, "pdf.immediateobj() allows max. 3 arguments");
+                pdf_end_obj(static_pdf);
+            } else {
+                pdf_begin_obj(static_pdf, k, OBJSTM_NEVER); /* not an object stream candidate! */
+                pdf_begin_dict(static_pdf);
+                if (n == first_arg + 2) {   /* write attr text */
+                    if (!lua_isstring(L, first_arg + 2)) /* or number (maybe only string as it's an attr) */
+                        luaL_error(L, "pdf.immediateobj() 3rd argument must be string");
+                    st3.s = lua_tolstring(L, first_arg + 2, &st3.l);
+                    pdf_out_block(static_pdf, st3.s, st3.l);
+                    if (st3.s[st3.l - 1] != '\n')
+                        pdf_out(static_pdf, '\n');
+                }
+                pdf_dict_add_streaminfo(static_pdf);
+                pdf_end_dict(static_pdf);
+                pdf_begin_stream(static_pdf);
+                if (lua_key_eq(st1_s, stream)) {
+                    pdf_out_block(static_pdf, st2.s, st2.l);
+                }  else if (lua_key_eq(st1_s, streamfile)) {
+                    buf.s = fread_to_buf(L, st2.s, &buf.l);
+                    pdf_out_block(static_pdf, (const char *) buf.s, buf.l);
+                    xfree(buf.s);
+                } else
+                    luaL_error(L, "pdf.immediateobj() invalid argument");
+                pdf_end_stream(static_pdf);
+                pdf_end_obj(static_pdf);
+            }
+            break;
+        default:
+            luaL_error(L, "pdf.immediateobj() allows max. 3 arguments");
     }
     lua_pushinteger(L, k);
     return 1;
@@ -193,23 +183,16 @@ static int table_obj(lua_State * L)
     attr.s = st.s = NULL;
     attr.l = 0;
     assert(lua_istable(L, 1));  /* t */
-
-    /* get object "type" */
-
     lua_key_rawgeti(type);
     if (lua_isnil(L, -1))       /* !vs t */
         luaL_error(L, "pdf.obj(): object \"type\" missing");
-    if (!lua_isstring(L, -1))   /* !vs t */
+    if (lua_type(L,-1) != LUA_TSTRING)   /* !vs t */
         luaL_error(L, "pdf.obj(): object \"type\" must be string");
     type = lua_tostring(L, -1);
-
     if (! (lua_key_eq(type, raw) || lua_key_eq(type, stream))) {
         luaL_error(L, "pdf.obj(): \"%s\" is not a valid object type", type);     /* i vs t */
     }
     lua_pop(L, 1);              /* t */
-
-    /* get optional "immediate" */
-
     lua_key_rawgeti(immediate);
     if (!lua_isnil(L, -1)) {    /* b? t */
         if (!lua_isboolean(L, -1))      /* !b t */
@@ -222,7 +205,7 @@ static int table_obj(lua_State * L)
 
     lua_key_rawgeti(objnum);
     if (!lua_isnil(L, -1)) {    /* vi? t */
-        if (!lua_isnumber(L, -1))       /* !vi t */
+        if (lua_type(L,-1) != LUA_TNUMBER)       /* !vi t */
             luaL_error(L, "pdf.obj(): \"objnum\" must be integer");
         k = (int) lua_tointeger(L, -1); /* vi t */
         check_obj_type(static_pdf, obj_type_obj, k);
@@ -245,7 +228,7 @@ static int table_obj(lua_State * L)
     if (!lua_isnil(L, -1)) {    /* attr-s? t */
         if (! lua_key_eq(type, stream))
             luaL_error(L, "pdf.obj(): \"attr\" key not allowed for non-stream object");
-        if (!lua_isstring(L, -1))       /* !attr-s t */
+        if (!lua_isstring(L, -1)) /* or number */      /* !attr-s t */
             luaL_error(L, "pdf.obj(): object \"attr\" must be string");
         if (immediate == 1) {
             attr.s = lua_tolstring(L, -1, &attr.l);     /* attr-s t */
@@ -262,7 +245,7 @@ static int table_obj(lua_State * L)
     if (!lua_isnil(L, -1)) {    /* vi? t */
         if (lua_key_eq(type, raw))
             luaL_error(L, "pdf.obj(): \"compresslevel\" key not allowed for raw object");
-        if (!lua_isnumber(L, -1))       /* !vi t */
+        if (lua_type(L, -1) != LUA_TNUMBER)       /* !vi t */
             luaL_error(L, "pdf.obj(): \"compresslevel\" must be integer");
         compress_level = (int) lua_tointeger(L, -1);    /* vi t */
         if (compress_level > 9)
@@ -309,20 +292,16 @@ static int table_obj(lua_State * L)
         if (!lua_isnil(L, -2)) {        /* file-s? string-s? t */
             /* from string */
             lua_pop(L, 1);      /* string-s? t */
-            if (!lua_isstring(L, -1))   /* !string-s t */
+            if (!lua_isstring(L, -1)) /* or number */   /* !string-s t */
                 luaL_error(L, "pdf.obj(): \"string\" must be string for raw object");
             if (immediate == 1) {
                 st.s = lua_tolstring(L, -1, &st.l);
                 pdf_out_block(static_pdf, st.s, st.l);
-                /* already in pdf_end_obj:
-                    if (st.s[st.l - 1] != '\n')
-                        pdf_out(static_pdf, '\n');
-                */
             } else
                 obj_obj_data(static_pdf, k) = luaL_ref(L, LUA_REGISTRYINDEX);   /* t */
         } else {
             /* from file */
-            if (!lua_isstring(L, -1))   /* !file-s nil t */
+            if (lua_type(L, -1) != LUA_TSTRING)   /* !file-s nil t */
                 luaL_error(L, "pdf.obj(): \"file\" name must be string for raw object");
             if (immediate == 1) {
                 st.s = lua_tolstring(L, -1, &st.l);     /* file-s nil t */
@@ -362,7 +341,7 @@ static int table_obj(lua_State * L)
         if (!lua_isnil(L, -2)) {        /* file-s? string-s? t */
             /* from string */
             lua_pop(L, 1);      /* string-s? t */
-            if (!lua_isstring(L, -1))   /* !string-s t */
+            if (!lua_isstring(L, -1)) /* or number */  /* !string-s t */
                 luaL_error(L, "pdf.obj(): \"string\" must be string for stream object");
             if (immediate == 1) {
                 st.s = lua_tolstring(L, -1, &st.l);     /* string-s t */
@@ -371,7 +350,7 @@ static int table_obj(lua_State * L)
                 obj_obj_data(static_pdf, k) = luaL_ref(L, LUA_REGISTRYINDEX);   /* t */
         } else {
             /* from file */
-            if (!lua_isstring(L, -1))   /* !file-s nil t */
+            if (lua_type(L, -1) != LUA_TSTRING)   /* !file-s nil t */
                 luaL_error(L, "pdf.obj(): \"file\" name must be string for stream object");
             if (immediate == 1) {
                 st.s = lua_tolstring(L, -1, &st.l);     /* file-s nil t */
@@ -398,9 +377,9 @@ static int orig_obj(lua_State * L)
     int k;
     const char *st_s = NULL ;
     n = lua_gettop(L);
-    if (n > 0 && lua_type(L, 1) == LUA_TNUMBER) {
+    if ((n > 0) && (lua_type(L, 1) == LUA_TNUMBER)) {
         first_arg++;
-        k=(int)lua_tonumber(L, 1);
+        k = (int) lua_tointeger(L, 1);
         check_obj_type(static_pdf, obj_type_obj, k);
         if (is_obj_scheduled(static_pdf, k) || obj_data_ptr(static_pdf, k) != 0)
             luaL_error(L, "pdf.obj() object in use");
@@ -416,14 +395,14 @@ static int orig_obj(lua_State * L)
         luaL_error(L, "pdf.obj() needs at least one argument");
         break;
     case 1:
-        if (!lua_isstring(L, first_arg))
+        if (!lua_isstring(L, first_arg)) /* or number */
             luaL_error(L, "pdf.obj() 1st argument must be string");
         break;
     case 2:
     case 3:
-        if (!lua_isstring(L, first_arg))
+        if (lua_type(L, first_arg) != LUA_TSTRING)
             luaL_error(L, "pdf.obj() 1st argument must be string");
-        if (!lua_isstring(L, first_arg + 1))
+        if (!lua_isstring(L, first_arg + 1)) /* or number */
             luaL_error(L, "pdf.obj() 2nd argument must be string");
         st_s = lua_tostring(L, first_arg);
         if (lua_key_eq(st_s, file)) {
@@ -432,7 +411,7 @@ static int orig_obj(lua_State * L)
             set_obj_obj_is_file(static_pdf, k);
         } else {
             if (n == first_arg + 2) {   /* write attr text */
-                if (!lua_isstring(L, -1))
+                if (!lua_isstring(L, -1)) /* or number */
                     luaL_error(L, "pdf.obj() 3rd argument must be string");
                 obj_obj_stream_attr(static_pdf, k) =
                     luaL_ref(Luas, LUA_REGISTRYINDEX);
@@ -488,12 +467,12 @@ static int l_reserveobj(lua_State * L)
     switch (n) {
     case 0:
         static_pdf->obj_count++;
-        pdf_last_obj =
-            pdf_create_obj(static_pdf, obj_type_obj, static_pdf->obj_ptr + 1);
+        pdf_last_obj = pdf_create_obj(static_pdf, obj_type_obj, static_pdf->obj_ptr + 1);
         break;
     case 1:
-        if (!lua_isstring(L, -1))
+        if (lua_type(L, -1) != LUA_TSTRING)
             luaL_error(L, "pdf.reserveobj() optional argument must be string");
+        st_s = luaL_checkstring(L, 1);
         if (lua_key_eq(st_s, annot)) {
             pdf_last_annot = pdf_create_obj(static_pdf, obj_type_annot, 0);
         } else {
@@ -527,137 +506,106 @@ static int l_registerannot(lua_State * L)
     return 0;
 }
 
-static int l_get_pdf_value(lua_State * L, int key)
-{
-    lua_rawgeti(L, LUA_REGISTRYINDEX, lua_key_index(pdf_data));
-    lua_gettable(L, LUA_REGISTRYINDEX);
-    /* [table] */
-    lua_rawgeti(L, LUA_REGISTRYINDEX, key);
-    /* [table] [key] */
-    lua_rawget(L,-2);
+/*
+
+# define valid_pdf_key ( \
+    lua_key_eq(s,pageresources) \
+    lua_key_eq(s,pageattributes) || \
+    lua_key_eq(s,pagesattributes) || \
+    lua_key_eq(s,catalog) || \
+    lua_key_eq(s,info) || \
+    lua_key_eq(s,names) || \
+    lua_key_eq(s,trailer) || \
+    lua_key_eq(s,xformresources) || \
+    lua_key_eq(s,xformattributes) || \
+    lua_key_eq(s,trailerid) \
+)
+
+*/
+
+#define l_get_pdf_value(key) \
+    lua_get_metatablelua(pdf_data); \
+    lua_key_rawgeti(key); \
     return 1;
-}
-static int l_get_pageresources(lua_State * L) {
-    return l_get_pdf_value(L,lua_key_index(pageresources));
-}
-static int l_get_pageattributes(lua_State * L) {
-    return l_get_pdf_value(L,lua_key_index(pageattributes));
-}
-static int l_get_pagesattributes(lua_State * L) {
-    return l_get_pdf_value(L,lua_key_index(pagesattributes));
-}
-static int l_get_catalog(lua_State * L) {
-    return l_get_pdf_value(L,lua_key_index(catalog));
-}
-static int l_get_info(lua_State * L) {
-    return l_get_pdf_value(L,lua_key_index(info));
-}
-static int l_get_names(lua_State * L) {
-    return l_get_pdf_value(L,lua_key_index(names));
-}
 
+static int l_get_pageresources  (lua_State * L) { l_get_pdf_value(pageresources); }
+static int l_get_pageattributes (lua_State * L) { l_get_pdf_value(pageattributes); }
+static int l_get_pagesattributes(lua_State * L) { l_get_pdf_value(pagesattributes); }
+static int l_get_catalog        (lua_State * L) { l_get_pdf_value(catalog); }
+static int l_get_info           (lua_State * L) { l_get_pdf_value(info); }
+static int l_get_names          (lua_State * L) { l_get_pdf_value(names); }
+static int l_get_trailer        (lua_State * L) { l_get_pdf_value(trailer); }
+static int l_get_xformresources (lua_State * L) { l_get_pdf_value(xformresources); }
+static int l_get_xformattributes(lua_State * L) { l_get_pdf_value(xformattributes); }
+static int l_get_trailerid      (lua_State * L) { l_get_pdf_value(trailerid); }
 
-static int l_get_trailer(lua_State * L) {
-    return l_get_pdf_value(L,lua_key_index(trailer));
-}
-
+/*
 
 static int getpdf(lua_State * L)
 {
-    /* [pdf table] [key] */
     const char *s ;
     if (lua_gettop(L) != 2) {
         return 0;
     }
-    if (lua_isstring(L, -1)) {
+    if (lua_type(L,-1) == LUA_TSTRING) {
         s =  lua_tostring(L, -1);
         if (lua_key_eq(s,h)) {
-            lua_pushnumber(L, static_pdf->posstruct->pos.h);
-	    return 1;
+            lua_pushinteger(L, static_pdf->posstruct->pos.h);
+            return 1;
         } else if (lua_key_eq(s,v)) {
-            lua_pushnumber(L, static_pdf->posstruct->pos.v);
-	    return 1;
-        } else if (
-                lua_key_eq(s,catalog) || lua_key_eq(s,info) || lua_key_eq(s,trailer) || lua_key_eq(s,names) ||
-                lua_key_eq(s,pageattributes) || lua_key_eq(s,pagesattributes) || lua_key_eq(s,pageresources)
-            ) {
-            lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_index(pdf_data));
-            lua_gettable(L, LUA_REGISTRYINDEX);
-            /* [pdf table] [key] [pdf.data table] */
+            lua_pushinteger(L, static_pdf->posstruct->pos.v);
+            return 1;
+        } else if (valid_pdf_key) {
+            lua_get_metatablelua(pdf_data);
             lua_replace(L, -3);
-            /* [pdf.data table] [key] */
-	    lua_rawget(L, -2);
-	    return 1;
+            lua_rawget(L, -2);
+            return 1;
         }
     }
     return 0;
 }
+*/
 
-
-
-
-static int l_set_pdf_value(lua_State * L, int key)
-{
-    if (lua_isstring(L, -1)) {
-        /* [value] */
-        lua_rawgeti(L, LUA_REGISTRYINDEX, lua_key_index(pdf_data));
-        lua_gettable(L, LUA_REGISTRYINDEX);
-        /* [value] [table]  */
-        lua_rawgeti(L, LUA_REGISTRYINDEX, key);
-        /* [value] [table] [key] */
-        lua_pushvalue(L, -3);
-        /* [table] [key] [value] */
-        lua_rawset(L,-3);
-    }
+#define l_set_pdf_value(key) \
+    if (lua_type(L,-1) == LUA_TSTRING) { \
+        lua_get_metatablelua(pdf_data); \
+        lua_rawgeti(L, LUA_REGISTRYINDEX, lua_key_index(key)); \
+        lua_pushvalue(L, -3); \
+        lua_rawset(L,-3); \
+    } \
     return 0;
-}
-static int l_set_pageresources(lua_State * L) {
-    return l_set_pdf_value(L,lua_key_index(pageresources));
-}
-static int l_set_pageattributes(lua_State * L) {
-    return l_set_pdf_value(L,lua_key_index(pageattributes));
-}
-static int l_set_pagesattributes(lua_State * L) {
-    return l_set_pdf_value(L,lua_key_index(pagesattributes));
-}
-static int l_set_catalog(lua_State * L) {
-    return l_set_pdf_value(L,lua_key_index(catalog));
-}
-static int l_set_info(lua_State * L) {
-    return l_set_pdf_value(L,lua_key_index(info));
-}
-static int l_set_names(lua_State * L) {
-    return l_set_pdf_value(L,lua_key_index(names));
-}
-static int l_set_trailer(lua_State * L) {
-    return l_set_pdf_value(L,lua_key_index(trailer));
-}
 
+static int l_set_pageresources  (lua_State * L) { l_set_pdf_value(pageresources); }
+static int l_set_pageattributes (lua_State * L) { l_set_pdf_value(pageattributes); }
+static int l_set_pagesattributes(lua_State * L) { l_set_pdf_value(pagesattributes); }
+static int l_set_catalog        (lua_State * L) { l_set_pdf_value(catalog); }
+static int l_set_info           (lua_State * L) { l_set_pdf_value(info); }
+static int l_set_names          (lua_State * L) { l_set_pdf_value(names); }
+static int l_set_trailer        (lua_State * L) { l_set_pdf_value(trailer); }
+static int l_set_xformresources (lua_State * L) { l_set_pdf_value(xformresources); }
+static int l_set_xformattributes(lua_State * L) { l_set_pdf_value(xformattributes); }
+static int l_set_trailerid      (lua_State * L) { l_set_pdf_value(trailerid); }
+
+/*
 
 static int setpdf(lua_State * L)
 {
-    /* [pdf table] [key] [value] */
     const char *s ;
     if (lua_gettop(L) != 3) {
         return 0;
     }
-    if (lua_isstring(L, -2)) {
-        s =  lua_tostring(L, -1);
-        if (
-                lua_key_eq(s,catalog) || lua_key_eq(s,info) || lua_key_eq(s,trailer) || lua_key_eq(s,names) ||
-                lua_key_eq(s,pageattributes) || lua_key_eq(s,pagesattributes) || lua_key_eq(s,pageresources)
-            ) {
-            lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_index(pdf_data));
-            lua_gettable(L, LUA_REGISTRYINDEX);
-            /* [pdf table] [key] [value] [pdf.data table] */
+    if (lua_type(L, -2) == LUA_TSTRING) {
+        s = lua_tostring(L, -1);
+        if (valid_pdf_key) {
+            lua_get_metatablelua(pdf_data);
             lua_replace(L, -4);
-            /* [pdf.data table] [key] [value] */
         }
     }
     lua_rawset(L, -3);
     return 0;
 }
 
+*/
 
 static int l_objtype(lua_State * L)
 {
@@ -685,7 +633,7 @@ static int l_mapfile(lua_State * L)
 {
     char *s;
     const char *st;
-    if (lua_isstring(L, -1) && (st = lua_tostring(L, -1)) != NULL) {
+    if ((lua_type(L,-1) == LUA_TSTRING) && (st = lua_tostring(L, -1)) != NULL) {
         s = xstrdup(st);
         process_map_item(s, MAPFILE);
         free(s);
@@ -697,7 +645,7 @@ static int l_mapline(lua_State * L)
 {
     char *s;
     const char *st;
-    if (lua_isstring(L, -1) && (st = lua_tostring(L, -1)) != NULL) {
+    if ((lua_type(L,-1) == LUA_TSTRING) && (st = lua_tostring(L, -1)) != NULL) {
         s = xstrdup(st);
         process_map_item(s, MAPLINE);
         free(s);
@@ -714,26 +662,26 @@ static int l_pageref(lua_State * L)
     if (n <= 0)
         luaL_error(L, "pdf.pageref() needs page number > 0");
     n = pdf_get_obj(static_pdf, obj_type_page, n, false);
-    lua_pushnumber(L, n);
+    lua_pushinteger(L, n);
     return 1;
 }
 
 static int l_getpos(lua_State * L)
 {
-    lua_pushnumber(L, static_pdf->posstruct->pos.h);
-    lua_pushnumber(L, static_pdf->posstruct->pos.v);
+    lua_pushinteger(L, static_pdf->posstruct->pos.h);
+    lua_pushinteger(L, static_pdf->posstruct->pos.v);
     return 2;
 }
 
 static int l_gethpos(lua_State * L)
 {
-    lua_pushnumber(L, static_pdf->posstruct->pos.h);
+    lua_pushinteger(L, static_pdf->posstruct->pos.h);
     return 1;
 }
 
 static int l_getvpos(lua_State * L)
 {
-    lua_pushnumber(L, static_pdf->posstruct->pos.v);
+    lua_pushinteger(L, static_pdf->posstruct->pos.v);
     return 1;
 }
 
@@ -748,12 +696,12 @@ static int l_getmatrix(lua_State * L)
         lua_pushnumber(L, m->e);
         lua_pushnumber(L, m->f);
     } else {
-        lua_pushnumber(L, 1);
-        lua_pushnumber(L, 0);
-        lua_pushnumber(L, 0);
-        lua_pushnumber(L, 1);
-        lua_pushnumber(L, 0);
-        lua_pushnumber(L, 0);
+        lua_pushinteger(L, 1);
+        lua_pushinteger(L, 0);
+        lua_pushinteger(L, 0);
+        lua_pushinteger(L, 1);
+        lua_pushinteger(L, 0);
+        lua_pushinteger(L, 0);
     }
     return 6 ;
 }
@@ -764,43 +712,438 @@ static int l_hasmatrix(lua_State * L)
     return 1 ;
 }
 
+static int l_get_lastlink(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_last_link));
+    return 1 ;
+}
+
+static int l_get_retval(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_retval));
+    return 1 ;
+}
+
+static int l_get_lastobj(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_last_obj));
+    return 1 ;
+}
+
+static int l_get_lastannot(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_last_annot));
+    return 1 ;
+}
+
+/* maybe:
+
+    get_fontname    : set_ff(i)  obj_info(static_pdf, pdf_font_num(ff))
+    get_fontobjnum  : set_ff(i)  pdf_font_num(ff)
+    get_fontsize    : font_size(i)
+    get_xformname   : obj_info(static_pdf, i)
+
+*/
+
+static int l_get_compress_level(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_compress_level));
+    return 1 ;
+}
+
+static int l_get_obj_compress_level(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_obj_compress_level));
+    return 1 ;
+}
+
+static int l_set_compress_level(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int c = (int) lua_tointeger(L, 1);
+        if (c<0)
+            c = 0 ;
+        else if (c>9)
+            c = 9 ;
+        set_pdf_compress_level(c);
+    }
+    return 0 ;
+}
+
+static int l_set_obj_compress_level(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int c = (int) lua_tointeger(L, 1);
+        if (c<0)
+            c = 0 ;
+        else if (c>9)
+            c = 9 ;
+        set_pdf_obj_compress_level(c);
+    }
+    return 0 ;
+}
+
+/* accuracy */
+
+static int l_get_decimal_digits(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_decimal_digits));
+    return 1 ;
+}
+
+static int l_set_decimal_digits(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int c = (int) lua_tointeger(L, 1);
+        if (c<0) {
+            c = 0 ;
+        }
+        set_pdf_decimal_digits(c);
+    }
+    return 0 ;
+}
+
+/* pk */
+
+static int l_get_pk_resolution(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_pk_resolution));
+    lua_pushinteger(L, (pdf_pk_fixed_dpi));
+    return 2 ;
+}
+
+static int l_set_pk_resolution(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int c = (int) lua_tointeger(L, 1);
+        if (c < 72) {
+            c = 72 ;
+        } else if (c > 8000) {
+            c = 8000 ;
+        }
+        set_pdf_pk_resolution(c);
+    }
+    if (lua_type(L, 2) == LUA_TNUMBER) {
+        set_pdf_pk_fixed_dpi(lua_tointeger(L, 1));
+    }
+    return 0 ;
+}
+
+/* pdf stuff */
+
+static int getpdffontname(lua_State * L)
+{
+    int c, ff ;
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        c = (int) lua_tointeger(L, 1);
+        pdf_check_vf(c);
+        if (!font_used(c))
+            pdf_init_font(static_pdf,c);
+        set_ff(c);
+        lua_pushinteger(L, (obj_info(static_pdf, pdf_font_num(ff))));
+    } else {
+        lua_pushnil(L);
+    }
+    return 1 ;
+}
+
+static int getpdffontobjnum(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int ff;
+        int c = (int) lua_tointeger(L, 1);
+        pdf_check_vf(c);
+        if (!font_used(c))
+            pdf_init_font(static_pdf,c);
+        set_ff(c);
+        lua_pushinteger(L, (pdf_font_num(ff)));
+    } else {
+        lua_pushnil(L);
+    }
+    return 1 ;
+}
+
+static int getpdffontsize(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int c = (int) lua_tointeger(L, 1);
+        lua_pushinteger(L, (font_size(c)));
+    } else {
+        lua_pushnil(L);
+    }
+    return 1 ;
+}
+
+/*
+
+static int getpdfpageref(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int c = (int) lua_tointeger(L, 1);
+        lua_pushinteger(L, (pdf_get_obj(static_pdf, obj_type_page, c, false)));
+    } else {
+        lua_pushnil(L);
+    }
+    return 1 ;
+}
+
+*/
+
+static int getpdfxformname(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int c = (int) lua_tointeger(L, 1);
+        check_obj_type(static_pdf, obj_type_xform, c);
+        lua_pushinteger(L, (obj_info(static_pdf, c)));
+    } else {
+        lua_pushnil(L);
+    }
+    return 1 ;
+}
+
+static int getpdfversion(lua_State * L)
+{
+    lua_pushinteger(L,1);
+    return 1 ;
+}
+
+static int getpdfminorversion(lua_State * L)
+{
+ /* lua_pushinteger(L,static_pdf->minor_version); */
+    lua_pushinteger(L,pdf_minor_version);
+    return 1 ;
+}
+
+static int setpdfminorversion(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        int c = (int) lua_tointeger(L, 1);
+        if ((c >= 0) && (c <= 9)) {
+            static_pdf->minor_version = c;
+            set_pdf_minor_version(c);
+        }
+    }
+    return 0 ;
+}
+
+static int setpdforigin(lua_State * L)
+{
+    int h = 0 ;
+    int v = 0 ;
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        h = (int) lua_tointeger(L, 1);
+        if (lua_type(L, 2) == LUA_TNUMBER) {
+            v = (int) lua_tointeger(L, 1);
+        } else {
+            v = h;
+        }
+    }
+    set_tex_extension_dimen_register(d_pdf_h_origin,h);
+    set_tex_extension_dimen_register(d_pdf_v_origin,v);
+    return 0 ;
+}
+
+static int getpdforigin(lua_State * L)
+{
+    lua_pushinteger(L,get_tex_extension_dimen_register(d_pdf_h_origin));
+    lua_pushinteger(L,get_tex_extension_dimen_register(d_pdf_v_origin));
+    return 2 ;
+}
+
+static int setpdfthreadmargin(lua_State * L) {
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        set_tex_extension_dimen_register(d_pdf_thread_margin,lua_tointeger(L, 1));
+    }
+    return 0;
+}
+
+static int setpdfdestmargin(lua_State * L) {
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        set_tex_extension_dimen_register(d_pdf_dest_margin,lua_tointeger(L, 1));
+    }
+    return 0;
+}
+
+static int setpdflinkmargin(lua_State * L) {
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        set_tex_extension_dimen_register(d_pdf_link_margin,lua_tointeger(L, 1));
+    }
+    return 0;
+}
+
+static int setpdfxformmargin(lua_State * L) {
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        set_tex_extension_dimen_register(d_pdf_xform_margin,lua_tointeger(L, 1));
+    }
+    return 0;
+}
+
+static int getpdfthreadmargin(lua_State * L) {
+    lua_pushinteger(L,get_tex_extension_dimen_register(d_pdf_thread_margin));
+    return 1;
+}
+
+static int getpdfdestmargin(lua_State * L) {
+    lua_pushinteger(L,get_tex_extension_dimen_register(d_pdf_dest_margin));
+    return 1;
+}
+
+static int getpdflinkmargin(lua_State * L) {
+    lua_pushinteger(L,get_tex_extension_dimen_register(d_pdf_link_margin));
+    return 1;
+}
+
+static int getpdfxformmargin(lua_State * L) {
+    lua_pushinteger(L,get_tex_extension_dimen_register(d_pdf_xform_margin));
+    return 1;
+}
+
+static int setpdfinclusionerrorlevel(lua_State * L) {
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        set_tex_extension_count_register(c_pdf_inclusion_errorlevel,lua_tointeger(L, 1));
+    }
+    return 0;
+}
+
+static int setpdfignoreunknownimages(lua_State * L) {
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        set_tex_extension_count_register(c_pdf_ignore_unknown_images,lua_tointeger(L, 1));
+    }
+    return 0;
+}
+
+static int getpdfinclusionerrorlevel(lua_State * L) {
+    lua_pushinteger(L,get_tex_extension_count_register(c_pdf_inclusion_errorlevel));
+    return 1;
+}
+
+static int getpdfignoreunknownimages(lua_State * L) {
+    lua_pushinteger(L,get_tex_extension_count_register(c_pdf_ignore_unknown_images));
+    return 1;
+}
+
+    static int l_set_suppress_optional_info(lua_State * L) {
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        set_tex_extension_count_register(c_pdf_suppress_optional_info,lua_tointeger(L, 1));
+    }
+    return 0;
+}
+
+static int l_get_suppress_optional_info(lua_State * L) {
+    lua_pushinteger(L,get_tex_extension_count_register(c_pdf_suppress_optional_info));
+    return 1;
+}
+
+static int newpdfcolorstack(lua_State * L)
+{
+    const char *s = NULL;
+    const char *l = NULL;
+    int literal_mode = 0; /* set_origin */
+    boolean page_start = false;
+    int id ;
+    if (lua_type(L,1) != LUA_TSTRING) {
+        luaL_error(L, "pdf.newcolorstack() expects a string as first argument");
+    }
+    s =	lua_tostring(L, 1);
+    if (lua_type(L,2) == LUA_TSTRING) {
+        l =	lua_tostring(L, 2);
+        if (lua_key_eq(l,origin)) {
+            literal_mode = 0;
+        } else if (lua_key_eq(l,page))  {
+            literal_mode = 1; /* direct_page */
+        } else if (lua_key_eq(l,direct)) {
+            literal_mode = 2; /* direct_always */
+        } else {
+            luaL_error(L, "invalid literal mode in pdf.newcolorstack()");
+        }
+    }
+    if (lua_isboolean(L, 3)) {
+        page_start = lua_toboolean(L, 3);
+    }
+    id = newcolorstack(s, literal_mode, page_start);
+    lua_pushinteger(L, id);
+    return 1 ;
+}
 
 static const struct luaL_Reg pdflib[] = {
-    {"immediateobj", l_immediateobj},
-    {"mapfile", l_mapfile},
-    {"mapline", l_mapline},
-    {"maxobjnum", l_maxobjnum},
-    {"obj", l_obj},
-    {"objtype", l_objtype},
-    {"pageref", l_pageref},
-    {"print", luapdfprint},
-    {"refobj", l_refobj},
-    {"registerannot", l_registerannot},
-    {"reserveobj", l_reserveobj},
-    {"getpos", l_getpos},
-    {"gethpos", l_gethpos},
-    {"getvpos", l_getvpos},
-    {"getmatrix", l_getmatrix},
-    {"hasmatrix", l_hasmatrix},
-    {"setcatalog", l_set_catalog},
-    {"setinfo", l_set_info},
-    {"setnames", l_set_names},
-    {"settrailer", l_set_trailer},
-    {"setpageresources", l_set_pageresources},
-    {"setpageattributes", l_set_pageattributes},
-    {"setpagesattributes", l_set_pagesattributes},
-    {"getcatalog", l_get_catalog},
-    {"getinfo", l_get_info},
-    {"getnames", l_get_names},
-    {"gettrailer", l_get_trailer},
-    {"getpageresources", l_get_pageresources},
-    {"getpageattributes", l_get_pageattributes},
-    {"getpagesattributes", l_get_pagesattributes},
-    {NULL, NULL}                /* sentinel */
+    { "gethpos", l_gethpos },
+    { "getvpos", l_getvpos },
+    { "obj", l_obj },
+    { "immediateobj", l_immediateobj },
+    { "refobj", l_refobj },
+    { "registerannot", l_registerannot },
+    { "reserveobj", l_reserveobj },
+    { "getpos", l_getpos },
+ /* { "pageref", getpdfpageref }, */
+    { "maxobjnum", l_maxobjnum },
+    { "pageref", l_pageref },
+    { "print", luapdfprint },
+    { "objtype", l_objtype },
+    { "getmatrix", l_getmatrix },
+    { "hasmatrix", l_hasmatrix },
+    { "setcatalog", l_set_catalog },
+    { "setinfo", l_set_info },
+    { "setnames", l_set_names },
+    { "settrailer", l_set_trailer },
+    { "setpageresources", l_set_pageresources },
+    { "setpageattributes", l_set_pageattributes },
+    { "setpagesattributes", l_set_pagesattributes },
+    { "setxformresources", l_set_xformresources },
+    { "setxformattributes", l_set_xformattributes },
+    { "settrailerid", l_set_trailerid },
+    { "getcatalog", l_get_catalog },
+    { "getinfo", l_get_info },
+    { "getnames", l_get_names },
+    { "gettrailer", l_get_trailer },
+    { "getpageresources", l_get_pageresources },
+    { "getpageattributes", l_get_pageattributes },
+    { "getpagesattributes", l_get_pagesattributes },
+    { "getxformresources", l_get_xformresources },
+    { "getxformattributes", l_get_xformattributes },
+    { "gettrailerid", l_get_trailerid },
+    { "getlastlink", l_get_lastlink },
+    { "getretval", l_get_retval },
+    { "getlastobj", l_get_lastobj },
+    { "getlastannot", l_get_lastannot },
+    { "getcompresslevel", l_get_compress_level },
+    { "getobjcompresslevel", l_get_obj_compress_level },
+    { "setcompresslevel", l_set_compress_level },
+    { "setobjcompresslevel", l_set_obj_compress_level },
+    { "getdecimaldigits", l_get_decimal_digits },
+    { "setdecimaldigits", l_set_decimal_digits },
+    { "getpkresolution", l_get_pk_resolution },
+    { "setpkresolution", l_set_pk_resolution },
+    { "getsuppressoptionalinfo", l_get_suppress_optional_info },
+    { "setsuppressoptionalinfo", l_set_suppress_optional_info },
+    /* moved from tex table */
+    { "fontname", getpdffontname },
+    { "fontobjnum", getpdffontobjnum },
+    { "fontsize", getpdffontsize },
+    { "xformname", getpdfxformname },
+    { "getversion", getpdfversion },
+    { "getminorversion", getpdfminorversion },
+    { "setminorversion", setpdfminorversion },
+    { "newcolorstack", newpdfcolorstack },
+    { "setorigin", setpdforigin },
+    { "getorigin", getpdforigin },
+    { "setthreadmargin", setpdfthreadmargin },
+    { "setdestmargin", setpdfdestmargin },
+    { "setlinkmargin", setpdflinkmargin },
+    { "setxformmargin", setpdfxformmargin },
+    { "getthreadmargin", getpdfthreadmargin },
+    { "getdestmargin", getpdfdestmargin },
+    { "getlinkmargin", getpdflinkmargin },
+    { "getxformmargin", getpdfxformmargin },
+    { "getinclusionerrorlevel", getpdfinclusionerrorlevel },
+    { "getignoreunknownimages", getpdfignoreunknownimages },
+    { "setinclusionerrorlevel", setpdfinclusionerrorlevel },
+    { "setignoreunknownimages", setpdfignoreunknownimages },
+    { "mapfile", l_mapfile },
+    { "mapline", l_mapline },
+    /* sentinel */
+    {NULL, NULL}
 };
 
-/**********************************************************************/
-
 int luaopen_pdf(lua_State * L)
 {
     lua_pushstring(L,"pdf.data");
@@ -808,15 +1151,15 @@ int luaopen_pdf(lua_State * L)
     lua_settable(L,LUA_REGISTRYINDEX);
     /* */
     luaL_register(L, "pdf", pdflib);
-    /* build meta table */
+    /*
     luaL_newmetatable(L, "pdf.meta");
     lua_pushstring(L, "__index");
     lua_pushcfunction(L, getpdf);
-    /* do these later, NYI */
     lua_settable(L, -3);
     lua_pushstring(L, "__newindex");
     lua_pushcfunction(L, setpdf);
     lua_settable(L, -3);
-    lua_setmetatable(L, -2);    /* meta to itself */
+    lua_setmetatable(L, -2);
+    */
     return 1;
 }
