@@ -44,7 +44,6 @@ typedef enum {
     start_page_number_callback, stop_page_number_callback,
     start_run_callback, stop_run_callback,
     define_font_callback,
-    token_filter_callback,
     pre_output_filter_callback,
     buildpage_filter_callback,
     hpack_filter_callback, vpack_filter_callback,
@@ -55,13 +54,18 @@ typedef enum {
     pre_linebreak_filter_callback,
     linebreak_filter_callback,
     post_linebreak_filter_callback,
+    append_to_vlist_filter_callback,
     mlist_to_hlist_callback,
     finish_pdffile_callback,
     finish_pdfpage_callback,
     pre_dump_callback,
     start_file_callback, stop_file_callback,
-    show_error_message_callback,show_lua_error_hook_callback,
-    pdf_stream_filter_callback,
+    show_error_message_callback, show_lua_error_hook_callback,
+    show_warning_message_callback,
+    hpack_quality_callback, vpack_quality_callback,
+    process_rule_callback,
+    insert_local_par_callback,
+    contribute_filter_callback,
     total_callbacks
 } callback_callback_types;
 
@@ -85,11 +89,12 @@ extern void get_saved_lua_boolean(int i, const char *name, boolean * target);
 extern void get_saved_lua_number(int i, const char *name, int *target);
 extern void get_saved_lua_string(int i, const char *name, char **target);
 
-extern void get_lua_boolean(const char *table, const char *name,
-                            boolean * target);
+extern void get_lua_boolean(const char *table, const char *name, boolean * target);
 extern void get_lua_number(const char *table, const char *name, int *target);
 extern void get_lua_string(const char *table, const char *name, char **target);
 
+extern int lua_reader_callback(int callback_id, pointer *buffloc);
+
 extern char *get_lua_name(int i);
 
 /* texfileio.c */
@@ -127,7 +132,7 @@ static const char *const filetypes_right[] = { "?", ")", "}", ">", ">", ">>" } ;
             (void) run_callback(report_id, "dS->",left,name); \
         } \
     } \
-} while (0) 
+} while (0)
 
 #define report_stop_file(right) do { \
     if (tracefilenames) { \
@@ -141,3 +146,7 @@ static const char *const filetypes_right[] = { "?", ")", "}", ">", ">", ">>" } ;
 } while (0)
 
 #endif
+
+#define normal_page_filter(A) lua_node_filter_s(buildpage_filter_callback,lua_key_index(A))
+#define checked_page_filter(A) if (!output_active) lua_node_filter_s(buildpage_filter_callback,lua_key_index(A))
+#define checked_break_filter(A) if (!output_active) lua_node_filter_s(contribute_filter_callback,lua_key_index(A))
