@@ -1,5 +1,5 @@
 /* linebreak.h
-   
+
    Copyright 2009 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
@@ -17,7 +17,6 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-
 #ifndef LINEBREAK_H
 #  define LINEBREAK_H
 
@@ -39,12 +38,14 @@ extern void ext_do_line_break(int paragraph_dir,
                               int tolerance,
                               scaled emergency_stretch,
                               int looseness,
+                              /*
                               int hyphen_penalty,
                               int ex_hyphen_penalty,
-                              int pdf_adjust_spacing,
+                              */
+                              int adjust_spacing,
                               halfword par_shape_ptr,
                               int adj_demerits,
-                              int pdf_protrude_chars,
+                              int protrude_chars,
                               int line_penalty,
                               int last_line_fit,
                               int double_hyphen_demerits,
@@ -54,18 +55,13 @@ extern void ext_do_line_break(int paragraph_dir,
                               int hang_after,
                               halfword left_skip,
                               halfword right_skip,
-                              int pdf_each_line_height,
-                              int pdf_each_line_depth,
-                              int pdf_first_line_height,
-                              int pdf_last_line_depth,
                               halfword inter_line_penalties_ptr,
                               int inter_line_penalty,
                               int club_penalty,
                               halfword club_penalties_ptr,
                               halfword widow_penalties_ptr,
                               int widow_penalty,
-                              int broken_penalty, halfword final_par_glue,
-                              halfword pdf_ignored_dimen);
+                              int broken_penalty, halfword final_par_glue);
 
 extern void get_linebreak_info(int *, int *);
 extern halfword find_protchar_left(halfword l, boolean d);
@@ -73,33 +69,33 @@ extern halfword find_protchar_right(halfword l, halfword r);
 
 /* skipable nodes at the margins during character protrusion */
 
-#  define cp_skipable(a) ((! is_char_node((a))) &&                        \
-                        ((type((a)) == ins_node)                        \
-                         || (type((a)) == mark_node)                    \
-                         || (type((a)) == adjust_node)                  \
-                         || (type((a)) == penalty_node)                 \
-                         || ((type((a)) == whatsit_node) &&             \
-                             (subtype((a)) != pdf_refximage_node) &&    \
-                             (subtype((a)) != pdf_refxform_node))       \
-                         /* reference to an image or XObject form */    \
-                         || ((type((a)) == disc_node) &&                \
-                             (vlink_pre_break(a) == null) &&            \
-                             (vlink_post_break(a) == null) &&           \
-                             (vlink_no_break(a) == null))               \
-                         /* an empty |disc_node| */                     \
-                         || ((type((a)) == math_node) &&                \
-                             (surround((a)) == 0))                      \
-                         || ((type((a)) == kern_node) &&                \
-                             ((width((a)) == 0) ||                      \
-                              (subtype((a)) == normal)))                \
-                         || ((type((a)) == glue_node) &&                \
-                             (glue_ptr((a)) == zero_glue))              \
-                         || ((type((a)) == hlist_node) &&               \
-                             (width((a)) == 0) &&                       \
-                             (height((a)) == 0) &&                      \
-                             (depth((a)) == 0) &&                       \
-                             (list_ptr((a)) == null))                   \
-                         ))
+#  define zero_dimensions(a) ( \
+    (width((a)) == 0) && \
+    (height((a)) == 0) && \
+    (depth((a)) == 0) \
+)
 
+#  define empty_disc(a) ( \
+    (vlink_pre_break(a) == null) && \
+    (vlink_post_break(a) == null) && \
+    (vlink_no_break(a) == null) \
+)
+
+#  define cp_skipable(a) ( (! is_char_node((a))) && ( \
+    ((type((a)) == glue_node) && (glue_is_zero((a)))) \
+ ||  (type((a)) == penalty_node) \
+ || ((type((a)) == disc_node) && empty_disc(a)) \
+ || ((type((a)) == kern_node) && ((width((a)) == 0) || (subtype((a)) == normal))) \
+ || ((type((a)) == rule_node) && zero_dimensions(a)) \
+ || ((type((a)) == math_node) && (surround((a)) == 0)) \
+ ||  (type((a)) == dir_node) \
+ || ((type((a)) == hlist_node) && (list_ptr((a)) == null) && zero_dimensions(a)) \
+ ||  (type((a)) == local_par_node) \
+ ||  (type((a)) == ins_node) \
+ ||  (type((a)) == mark_node) \
+ ||  (type((a)) == adjust_node) \
+ ||  (type((a)) == boundary_node) \
+ ||  (type((a)) == whatsit_node) \
+) )
 
 #endif
