@@ -1,5 +1,5 @@
 /* texmath.h
-   
+
    Copyright 1996-2006 Han The Thanh <thanh@pdftex.org>
    Copyright 2006-2009 Taco Hoekwater <taco@luatex.org>
 
@@ -91,17 +91,6 @@ void unsave_math_data(int lvl);
   or |denom_style|).
 */
 
-typedef enum {
-    display_style = 0,          /* |subtype| for \.{\\displaystyle} */
-    cramped_display_style,      /* |subtype| for \.{\\crampeddisplaystyle} */
-    text_style,                 /* |subtype| for \.{\\textstyle} */
-    cramped_text_style,         /* |subtype| for \.{\\crampedtextstyle} */
-    script_style,               /* |subtype| for \.{\\scriptstyle} */
-    cramped_script_style,       /* |subtype| for \.{\\crampedscriptstyle} */
-    script_script_style,        /* |subtype| for \.{\\scriptscriptstyle} */
-    cramped_script_script_style,        /* |subtype| for \.{\\crampedscriptscriptstyle} */
-} math_style_subtypes;
-
 extern const char *math_style_names[];
 
 #  define cramped 1             /* add this to an uncramped style if you want to cramp it */
@@ -160,6 +149,8 @@ typedef enum {
     math_param_fraction_denom_vgap,
     math_param_fraction_denom_down,
     math_param_fraction_del_size,
+    math_param_skewed_fraction_hgap,
+    math_param_skewed_fraction_vgap,
     math_param_limit_above_vgap,
     math_param_limit_above_bgap,
     math_param_limit_above_kern,
